@@ -19,7 +19,7 @@ propOdds <- function(time,status,X,
 
   op <- nlminb(theta,function(x) U(x)^2)
   uu <- U(op$par,TRUE)
-  du <- grad(U,op$par)
+  du <- numDeriv::grad(U,op$par)
   return(list(theta=op$par, sd=(mean(uu^2)/du^2/ncluster)^0.5))
 }
                   
